package example;

import javax.annotation.Resource;
import javax.persistence.*;
import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;

/**
 * Serves as a base class for other servlets that use EntityManager
 */
public abstract class BaseServlet extends HttpServlet {

    @PersistenceUnit(unitName="pu1")
    private EntityManagerFactory emf;

    private static final String EM = "em";

    /** 
     * Works both in containers that support JPA as well as the ones that don't
     */
    protected EntityManager createEntityManager(HttpServletRequest req) {
	/*
	 * Because it stores em in HttpSession, this web app won't work
	 * when simultaneous requests are made from multi-frame html page
	 */
	HttpSession session = req.getSession();
	EntityManager em = EntityManager.class.cast(session.getAttribute(EM));
	if (em == null) {
            // Create an EM here so that only this thread uses it.
            em = emf.createEntityManager();
	    session.setAttribute(EM, em);
	}
        return em;
    }

    protected void closeEntityManager(HttpServletRequest req) {
	    HttpSession session = req.getSession();
	    EntityManager.class.cast(session.getAttribute(EM)).close();
	    session.removeAttribute(EM);
    }

    public void  init( ServletConfig config) throws ServletException {
        super.init(config);
	if (emf == null) {
	    // no need to synchronize as init is called only once for a Servlet
	    log("No JPA support, switch to SE mode");
	    emf = Persistence.createEntityManagerFactory("pu1");
	}
    }
    
    public void destroy() {
        emf.close();
	super.destroy();
    }
}
